/**
 * External dependencies
 */
import { decodeEntities } from "@wordpress/html-entities";
import { __ } from "@wordpress/i18n";
import { registerPaymentMethod } from "@woocommerce/blocks-registry";
import { getSibsCardServerData } from "./sibs-utils";

const Content = (props) => {
	const [sibs_stg_oneclick_customer_acceptor, setChecked] =
		React.useState("");
	const oneclicklabel = getSibsCardServerData()?.checkoutLabel || "";
	const { eventRegistration, emitResponse } = props;
	const { onPaymentProcessing } = eventRegistration;
	const handleChange = (event) => {
		setChecked(event.target.checked);
	};

	if (getSibsCardServerData()?.checkoutLabel) {
		React.useEffect(() => {
			const customerAcceptor = onPaymentProcessing(async () => {
				return {
					type: emitResponse.responseTypes.SUCCESS,
					meta: {
						paymentMethodData: {
							sibs_stg_oneclick_customer_acceptor,
						},
					},
				};
			});
			return () => {
				customerAcceptor();
			};
		}, [
			emitResponse.responseTypes.ERROR,
			emitResponse.responseTypes.SUCCESS,
			onPaymentProcessing,
			sibs_stg_oneclick_customer_acceptor,
		]);
		return (
			<div style={{ display: "flex", alignItems: "flex-start" }}>
				<input
					type="checkbox"
					name="sibs_stg_oneclick_customer_acceptor"
					className="wc-block-components-checkbox__input"
					onChange={handleChange}
					checked={
						sibs_stg_oneclick_customer_acceptor === false
							? true
							: sibs_stg_oneclick_customer_acceptor
					}
					id="sibs_stg_oneclick_customer_acceptor"
					style={{ display: "inline", marginRight: "5px" }}
				/>
				<label
					htmlFor="sibs_stg_oneclick_customer_acceptor"
					className="wc-block-components-checkbox__label"
				>
					<span dangerouslySetInnerHTML={{ __html: oneclicklabel }} />
				</label>
			</div>
		);
	} else {
		return "";
	}
};

const Label = () => {
	const logos = getSibsCardServerData()?.logos;
	const logo = [];
	var title = decodeEntities(getSibsCardServerData()?.title);

	logos.forEach((data) => {
		var url = getSibsCardServerData()?.logos_url + data + ".svg";
		logo.push(
			<img
				src={url}
				style={{ display: "inline", paddingRight: "5px" }}
			/>,
		);
	});

	return (
		<div className="wc-block-components-payment-method-label wc-block-components-payment-method-label--with-icon">
			{" "}
			{logo} <span> {title} </span>
		</div>
	);
};

registerPaymentMethod({
	name: "sibs_stg_cc",
	label: <Label />,
	ariaLabel: __("Sibs payment method", "sibs-woo-stg"),
	canMakePayment: () => true,
	content: <Content />,
	edit: <Content />,
	supports: {
		features: getSibsCardServerData()?.supports ?? [],
	},
});
