<?php

/**
 * sibs Admin Page.
 */
require_once plugin_dir_path(realpath(__DIR__ . '/')) . '/vendor/autoload.php';

use Sibs2\sibsConfig;
use Sibs2\sibsConstants;
use Sibs2\sibsFunctions;
use Sibs2\sibsLogger;
use Sibs2\sibsPayments;

function sibs_admin_menu()
{
    add_submenu_page(
        'woocommerce',
        'SIBS ' . __('General Settings', sibsConstants::SIBS_TRANSLATE),
        'SIBS ' . __('General Settings', sibsConstants::SIBS_TRANSLATE),
        'manage_options',
        'sibs',
        'sibs_page_content',
        3
    );
}

add_action('admin_menu', 'sibs_admin_menu');

function successMessage($saved = false)
{
    ?>
    <?php if (isset($_POST['sibs_save']) && $saved) { ?>
        <div class="sibs-notice notice notice-success">
            <p><strong><?= __('Settings saved', sibsConstants::SIBS_TRANSLATE) ?></strong></p>
        </div>
    <?php } ?>
<?php
}

function download_onboarding_file()
{
    $domain_onboarding_file_name        = 'apple-developer-merchantid-domain-association';
    $domain_onboarding_file_directory   = '.well-known';
    $onboarding_file_dir                = untrailingslashit(ABSPATH) . '/' . $domain_onboarding_file_directory;
    $onboarding_file_path               = $onboarding_file_dir . '/' . $domain_onboarding_file_name;

    sibsLogger::debug('_FILES: ' . sibsLogger::prettify($_FILES), sibsConstants::SIBS_ADMIN_LOG);

    if (isset($_FILES[sibsConfig::SIBS_GENERAL_CONFIG . '_merchant_domain_association'])) {
        // code...
        sibsLogger::debug('_FILES: TRUE', sibsConstants::SIBS_ADMIN_LOG);

        try {
            if (! file_exists($onboarding_file_dir) && ! mkdir($onboarding_file_dir, 0755)) {
                sibsLogger::debug(__('Can not on-boarding Apple Pay merchant: Can not create directory', sibsConstants::SIBS_TRANSLATE), sibsConstants::SIBS_ADMIN_LOG);

                return false;
            }

            global $wp_filesystem;

            if (empty($wp_filesystem)) {
                require_once ABSPATH . '/wp-admin/includes/file.php';
                WP_Filesystem();
            }

            if (! move_uploaded_file($_FILES[sibsConfig::SIBS_GENERAL_CONFIG . '_merchant_domain_association']['tmp_name'], $onboarding_file_path)) { // phpcs:ignore
                sibsLogger::debug(__('Can not on-boarding Apple Pay merchant: Can not locate on-boarding file', sibsConstants::SIBS_TRANSLATE), sibsConstants::SIBS_ADMIN_LOG);

                return false;
            }

            return true;
        } catch (Exception $e) {
            sibsLogger::error($e->getMessage(), sibsConstants::SIBS_ADMIN_LOG);

            return false;
        }
    }
}

function sibs_page_sidebar()
{
    ?>
    <div class="sibs-sidebar div2">
        <h2>
            <?= __('System Informations', sibsConstants::SIBS_TRANSLATE) ?>
        </h2>
        <?= sibsPayments::version_description() ?>
        <hr/>
        <h2>
            <?= __('Decryption Informations', sibsConstants::SIBS_TRANSLATE) ?>
        </h2>
            <?= sibsPayments::decrypt_description() ?>
        <hr/>
        <h2>
            <?= __('Other Informations', sibsConstants::SIBS_TRANSLATE) ?>
        </h2>
        <div class='report_feedback'>
            <?= sibsPayments::other_description() ?>
        </div>
        <hr/>
        <h2>
            <?= __('Browser Informations', sibsConstants::SIBS_TRANSLATE) ?>
        </h2>
        <?php
            $browser = new Sibs2\sibsBrowser($_SERVER['HTTP_USER_AGENT']);
    echo $browser->__toString();

    if ($browser->getBrowser() == Sibs2\sibsBrowser::BROWSER_SAFARI) {
        echo __('You have Safari', sibsConstants::SIBS_TRANSLATE);
    }
    ?>
    </div>
    <?php
}

function sibs_page_content()
{
    $saved = (isset($_POST['sibs_save'])) ? sibsPayments::saveData($_POST) : false;

    $environment_file = sibsFunctions::get_environment();
    $environment      = $environment_file['environment'];

    $languages = sibsConstants::SIBS_LANGUAGES;

    download_onboarding_file();

    ?>
    <script type="text/javascript">
       urlAPISIBS = "<?= plugins_url(sibsConstants::SIBS_TRANSLATE); ?>";
       urlStore = "<?= site_url()?>";
       prefix = "<?= sibsConstants::SIBS_PLUGIN_PREFIX . '' ?>";
       ambiente = "<?= implode(',', $environment)?>";
    </script>

    <div class="sibs_header">
        <div>
            <img src="<?= esc_attr(plugins_url(sibsConstants::SIBS_TRANSLATE . '/assets/images/logo.png', )) ?>" alt="Sibs">
        </div>
        <div>
            <h1>SIBS <?= __('General Settings', sibsConstants::SIBS_TRANSLATE) ?></h1>
        </div>
    </div>
    <?php successMessage($saved); ?>
    
    <div id="divSibsRadioConfiguration">
        <div class="toggle"> \
            <input type="radio" name="sibsconfigurationRadio" value="Manually" id="sizeDimensions" checked="checked" /> \
            <label for="sizeDimensions"><?= __('Manual Configuration', sibsConstants::SIBS_TRANSLATE) ?></label> \
            <input type="radio" name="sibsconfigurationRadio" value="Automatic" id="sizeWeight" /> \
            <label for="sizeWeight"><?= __('Automatic Configuration', sibsConstants::SIBS_TRANSLATE) ?></label> \
        </div>
    </div>

    <div class="sibs-settings">
        <form name="sibs-settings" method="POST" action="" class="div1" enctype="multipart/form-data">
            <div class="clear"></div>
            <table class="form-table" role="presentation">
                <tbody>
                    <tr>
                        <th scope="row">
                            <label for="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_merchant_no' ?>">
                                <?= __('Terminal code', sibsConstants::SIBS_TRANSLATE) . ' * ' ?>
                            </label>
                        </th>
                        <td>
                            <input class="regular-text" type="text"
                                id="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_merchant_no' ?>"
                                name="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_merchant_no' ?>"
                                value="<?= esc_attr(sibsConfig::getTerminalId()) ?>"
                                required />
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">
                            <label for="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_bearer_token' ?>">
                                <?= __('Bearer Token', sibsConstants::SIBS_TRANSLATE) . ' * ' ?>
                            </label>
                        </th>
                        <td>
                            <textarea class="regular-text" type="text"
                                id="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_bearer_token' ?>"
                                name="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_bearer_token' ?>"
                                rows="<?= ! empty(sibsConfig::getToken()) ? '12' : '4' ?>"
                                required><?= esc_attr(sibsConfig::getToken()) ?></textarea>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">
                            <label for="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_client_id' ?>">
                                <?= __('Client ID', sibsConstants::SIBS_TRANSLATE) . ' * ' ?>
                            </label>
                        </th>
                        <td>
                            <input class="regular-text" type="text"
                                id="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_client_id' ?>"
                                name="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_client_id' ?>"
                                value="<?= esc_attr(sibsConfig::getClientId()) ?>"
                                required />
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">
                            <label for="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_environment' ?>">
                                <?= __('SIBS Environment', sibsConstants::SIBS_TRANSLATE) ?>
                            </label>
                        </th>
                        <td>
                            <select id="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_environment' ?>"
                                name="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_environment' ?>">
                                <?php
                                    foreach ($environment as $key => $value) {
                                        $selected = ($key == sibsConfig::getEnvironment()) ?
                                            'selected' :
                                            '';
                                        ?>
                                            <option value="<?= $key ?>" <?= $selected ?>><?= $value ?></option>;
                                        <?php
                                    }
    ?>
                            </select>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">
                            <label for="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_language' ?>">
                                <?= __('Form Language', sibsConstants::SIBS_TRANSLATE) ?>
                            </label>
                        </th>
                        <td> 
                            
                            <select id="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_language' ?>"
                                name="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_language' ?>">
                                <?php
        foreach ($languages as $key => $value) {
            $selected = ($key == sibsConfig::getLanguageDefault()) ?
                'selected' :
                '';
            ?>
                                            <option value="<?= $key ?>" <?= $selected ?>><?= $value ?></option>;
                                        <?php
        }
    ?>
                            </select>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">
                            <label for="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_webhook_secret' ?>">
                                <?= __('SIBS WebHook Secret', sibsConstants::SIBS_TRANSLATE) ?>
                            </label>
                        </th>
                        <td>
                            <input class="regular-text" type="text"
                                id="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_webhook_secret' ?>"    
                                name="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_webhook_secret' ?>"
                                value="<?= esc_attr(sibsConfig::getWebhookSecret()) ?>">
                            <p>
                                <?= sibsPayments::webhook_description() ?>
                            </p>
                        </td>
                    </tr> 
                    <tr>
                        <th scope="row">
                            <span>
                                <?= __('Onboard shop domain for Apple Pay', sibsConstants::SIBS_TRANSLATE) ?>
                            </span>
                        </th>
                        <td>
                            <?= __('To learn more about how to do this, visit our', sibsConstants::SIBS_TRANSLATE) ?> 
                            <a href="https://www.docs.pay.sibs.com/payment-methods/apple-pay/" target="_blank">
                                <?= __('documentation', sibsConstants::SIBS_TRANSLATE) ?>
                            </a>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">
                            <label for="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_merchant_domain_association' ?>">
                                <?= __('MerchantId Domain Association', sibsConstants::SIBS_TRANSLATE) . ' * ' ?>
                            </label>
                        </th>
                        <td>
                            <input type="file" 
                                id="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_merchant_domain_association' ?>"
                                name="<?= sibsConfig::SIBS_GENERAL_CONFIG . '_merchant_domain_association' ?>" />
                        </td>
                    </tr> 
                </tbody>
            </table>
            <p>
                <input class="button button-primary savesibs" type="submit" 
                    name="sibs_save" value="<?= __('Save Changes', sibsConstants::SIBS_TRANSLATE) ?>">
            </p>
        </form>
        <?= sibs_page_sidebar() ?>
    </div>

    <div id="sibsConfigurationWizard">
        <div id="wizard" class="wizard">
            <h2><?= __('Login', sibsConstants::SIBS_TRANSLATE) ?></h2>
            <section>
                <?= sibsPayments::sibs_page_content_wizard_login() ?>
            </section>
            <h2><?= __('Configure Terminal', sibsConstants::SIBS_TRANSLATE) ?></h2>
            <section>
                <?= sibsPayments::sibs_page_content_wizard_terminal() ?>
            </section>
            <h2><?= __('Webhooks', sibsConstants::SIBS_TRANSLATE) ?></h2>
            <section>
                <?= sibsPayments::sibs_page_content_wizard_webhooks() ?>
            </section>
        </div>
    </div>

    <div class="lds-ring"></div>
<?php
}

?>